#!/usr/bin/perl -w

# Spinner Wiki - upload.pl
#        Copyright (C) 2000 Bruce Dodson (bdodson@esricanada.com)
#
# Based on eUpload

################################################################################
# eUpload                                      Version 1.0                     #
# Copyright 2000 Frederic TYNDIUK (FTLS)       All Rights Reserved.            #
# E-Mail: tyndiuk@ftls.org                     Script License: GPL             #
# Created  01/01/2000                          Last Modified 01/01/2000        #
# Scripts Archive at:                          http://www.ftls.org/cgi/        #
################################################################################
########################## license & copyright header ##########################
#                                                                              #
#                     Copyright (c) 1999 by TYNDIUK Frederic                   #
#                                                                              #
#       This program is free software; you can redistribute it and/or          #
#       modify it under the terms of the GNU General Public License as         #
#       published by the Free Software Foundation; either version 2 of         #
#       the License, or (at your option) any later version.                    #
#                                                                              #
#       This program is distributed in the hope that it will be useful,        #
#       but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#       GNU General Public License for more details.                           #
#                                                                              #
#       You should have received a copy of the GNU General Public License      #
#       along with this program in the file 'COPYING'; if not, write to        #
#       the Free Software Foundation, Inc., 59 Temple Place - Suite 330,       #
#       Boston, MA 02111-1307, USA, or contact the author:                     #
#                                                                              #
#                                   TYNDIUK Frederic <tyndiuk@ftls.org>        #
#                                            <http://www.ftls.org/>            #
#                                                                              #
######################## end license & copyright header ########################
################################################################################

# Modified by Bruce Dodson to use as part of Spinner Wiki
#
# Although it bears little resemblance to eUpload now (dropped
# most of the UI code, all of the account management, and changed
# much of the upload code), eUpload was extremely useful to me
# in getting this working quickly.  Highly recommended.


use strict;

use CGI;
use URI::Escape;
use wiki;


# Necessary Variables:                                                         #
my $MAX_SIZE_UPLOAD = $wiki::uploadMaxSize;
my $DEFAULT_UPLOAD_DIR = $wiki::uploadDir;
my $DEFAULT_UPLOAD_URL = $wiki::uploadUrl;


# File sizes are limited to $MAX_SIZE_UPLOAD (0 No limit), larger files will
# return an error.


# some of these values are still used; many others are not.
my %NAME_TITLE  = (
    'common_admin' => "eUpload, administrative display",
    'common_member' => "eUpload, upload display",
    'error_form' => 'Error : Incomplet form', 
    'manage_users' => 'Manage Users', 
    'edit_user' => "Edit 'Value_login' user", 
    'add_user' => 'Add a user',
    'user_saved' => "User 'Value_login' saved", 
    'user_added' => "New user 'Value_login' added", 
    'user_deleted' => "User 'Value_new_login' deleted",
    'enter_pass' => 'Enter your password', 
    'invalid_pass' => 'INVALID PASSWORD',
    'EU_BadFN' => "Error: Bad Name 'Value_FileName'",
    'EU_FExist' => "Error: File 'Value_FileName' exists, and this CGI doesn't think you should be allowed to overwrite it!  How un-Wiki-like!  Please delete the existing file, and try again.",
    'EU_Size' => "Error: Could not upload file: 'Value_FileName'",
    'Upload_Succes' => 'File uploaded successfully!',
    'Upload_Succes_txt' => "'Value_FileName' (Value_Size bytes) was saved to <a href=\"Value_URL\">Value_URL</a>."
);








if ($MAX_SIZE_UPLOAD) { 
    $CGI::POST_MAX=1024 * 1024 * $MAX_SIZE_UPLOAD; 
}

my $query = new CGI;


if ($query->request_method() eq 'POST' and length($query->param('upload'))>0) {
    doUpload($query, $DEFAULT_UPLOAD_DIR, $DEFAULT_UPLOAD_URL);

} elsif ($query->param('success')) {
    writeSuccessPage();

} elsif ($query->request_method() eq 'GET') {
    my $pathInfo= $query->path_info();

    if ($pathInfo =~ /^[\/].+/) {
        print $query->redirect($wiki::uploadUrl . $pathInfo);
    } else {
        writeUploadPage();
    }
    
}






sub redirectSuccessPage {
    my ($filename,$size,$time) = @_;
    $filename = uri_escape($filename);
    print $query->redirect(
        $wiki::uploadUrl . "/?success=$filename&size=$size&time=$time"
    );
}

sub writeSuccessPage {
    my ($filename,$size,$time) = @_;
    if (!$filename) {
        $filename = $query->param('success');
        $size = $query->param('size');
        $time = $query->param('time');
    }

    local $_ = $NAME_TITLE{'Upload_Succes_txt'};
    s/Value_FileName/$filename/ig;
    s/Value_Size/$size/ig;
    s/Value_Time/$time/ig;
    s/Value_URL/$wiki::uploadUrl\/$filename/ig;
    writeUploadPage("<big><b>$NAME_TITLE{Upload_Succes}</b></big> $_");
}


sub writeUploadPage {
    my $message = shift;
    #$message .= '<br>' if $message;

    print "Content-type: text/html\n\n";

    my $tmpl = wiki::readTemplate("upload");
    $tmpl = wiki::handleCommonTags(wiki::readTemplate("upload"));

    my $maxSize;
    if ($wiki::uploadMaxSize > 1) {
        $maxSize = $wiki::uploadMaxSize . " MB";
    } elsif ($wiki::uploadMaxSize > 0) {
        $maxSize = ($wiki::uploadMaxSize * 1024) . " KB";
    } else {
        $maxSize = "Not Set";
    }
    $tmpl =~ s/\$UPLOAD_MAX_SIZE\$/$maxSize/g;


    my ($before, $text, $after) = split( /\$REPEAT\$/, $tmpl);

    $before =~ s/\$MESSAGE\$/$message/g;
    $after =~ s/\$MESSAGE\$/$message/g;

    print $before;

    my @index = sort(<$DEFAULT_UPLOAD_DIR/*>);
    my $item;
    foreach $item ( @index ) {
        my $readOnly = !(-w $item);
        my $size = sprintf("%.1f",(-s $item) / 1024);
        $item = substr($item,length($DEFAULT_UPLOAD_DIR)+1);
        my ($itemBase, $itemExt) = split(/\.(?!.*\.)/,$item, 2);
        my $line = $text;
        $line =~ s/\$FILENAME\$/$item/go;
        $line =~ s/\$FILESIZE\$/$size/go;
        $line =~ s/\$FILEBASE\$/$itemBase/go;
        $line =~ s/\$FILEEXT\$/$itemExt/go;
        
        my $fileNote = $readOnly ? '(read-only)' : '';
        $line =~ s/\$FILENOTE\$/$fileNote/go;

        print $line;
    }
    print $after;
}



sub doUpload {
    my ($query, $upload_dir, $upload_url) = @_;
    my ($file_query, $file_name, $size, $buff, $time, $bytes_count);
    $size = $bytes_count =0;
    $_ = $file_query = $query->param('upload');
    s/^\w://;

    if (/([^\/\\]+)$/) {        # select the part after the last slash
        $_ = $1;
        s/\.\.+/\./g;           # kill any attempts to write up a directory.
        s/[\s_:\[\]\*\?\&]+/_/g;  # turn spaces and special chars into underline
        s/^\./_/;		# turn a leading dot into underline
        $file_name = lc($_);    # force lower case
    }
    
    if ((!$file_name) || ($file_name =~ /[\\\/\:]/)) {
        $_ = $NAME_TITLE{'EU_BadFN'};
        s/Value_FileName/$file_name/ig;
        reportError($_, 1);
    }

    # Just in case two people are uploading the same file at
    # the same time, make sure to generate a unique filename 
    # for each upload.
    my $work_file = "$upload_dir/.transfer_${file_name}";
    for (my $i=0 ; -e $work_file ; ++$i) {
        $work_file = "$upload_dir/.transfer${i}_${file_name}";
    }

    # It seems un-Wiki-like to refuse to overwrite a file...
    # if (-e "$upload_dir/$file_name") {
    #	$_ = $NAME_TITLE{'EU_FExist'};
    #	s/Value_FileName/$file_name/ig;
    #	reportError($_, 1);
    #}
    
    open(FILE,">$work_file") or reportError("Error opening file $file_name for writing, error $!", 1);
    binmode FILE;
    $time=time();
    while ($bytes_count = read($file_query,$buff,2096)) {
        $size += $bytes_count;
        print FILE $buff;
    }
    close(FILE);

    if ((stat "$work_file")[7] <= 0) {
   	unlink("$work_file");
        $_ = $NAME_TITLE{'EU_Size'};
        s/Value_FileName/$file_name/ig;
        reportError($_, 1);
    } else {
        rename($work_file,"$upload_dir/$file_name") or 
        reportError("Error: could not create/replace $file_name!", 1);

        $time = time - $time;

        redirectSuccessPage($file_name, $size, $time);
    }
}



sub reportError {
    my($errortext, $exit) = @_;
    $errortext =~ s/^\s*Error\:/<big><b>Error:<\/b><\/big>/;
    writeUploadPage("<font color=\"red\">$errortext</font>");
    if ($exit) { exit; }
}
